#include "skydome.h"
#include "../renderer.h"

skydome::skydome(float radius, int delta_theta, int delta_phi)
{
/* default values */
	set_atmosphere( color(0.125f, 0.125f, 1.0f), 1.0f, 0.9f );
	set_haze( color(0.9f, 0.9f, 0.9f), 0.2f, 0.8f );
	set_decay( color(0.9f, 0.4f, 0.4f), 0.2f, 0.9f );
	set_sun( color(1.0f, 1.0f, 1.0f), 1.0f, polar3f(45.0f, 45.0f, 0.0f) );

/* global colors calculate by skydome */
	m_sun_color = color( 0, 0, 0 );
	m_sun_flare_color = color( 0, 0, 0 );
	m_fog_color = color( 0, 0, 0 );
	m_cloud_color = color( 0, 0, 0 );

/* generate skydome */
	m_indices = NULL;
	m_vertices = NULL;

	m_delta_theta = delta_theta;
	m_delta_phi = delta_phi;
	m_radius = radius;

	generate_dome();
}

skydome::~skydome()
{
	if(m_indices!=NULL) delete[] m_indices;
	if(m_vertices!=NULL) delete[] m_vertices;
}

void skydome::generate_dome()
{
	float theta, phi;
	float needed, needed_next;
	int i,n = 0;
	int tempn;

	// Generate the dome
	int max_num_vertices=(int)((360/m_delta_theta)*(90/m_delta_phi)*2)+1;
	m_num_vertices=0;

	m_indices = new unsigned short[max_num_vertices];
	m_vertices = new vertex_color_polar[max_num_vertices];

	for (phi=90; phi >= m_delta_phi; phi -= m_delta_phi)
	{
		needed = (float)((int)((360/m_delta_theta)*wCosf(phi))+1);
		needed_next = (float)((int)((360/m_delta_theta)*wCosf(phi-(float)m_delta_phi))+1);

		for(i=0; i<needed_next; i++)
		{
			theta=360*(((int)(needed*(i/needed_next)))/needed);
			tempn=m_num_vertices+((int)(needed*(i/needed_next)));

			m_vertices[tempn].m_polar = polar3f( phi, theta, m_radius );
			m_vertices[tempn].m_position=vector3f( polar3f( phi, theta, m_radius ) );
			m_vertices[tempn].m_color=color(0,0,0);

			m_indices[n]=tempn;

			n++;

			theta=360*(i/needed_next);
			tempn=m_num_vertices+(int)needed+i;

			// Calculate the vertex at phi-m_delta_phi, theta
			m_vertices[tempn].m_polar = polar3f( phi-m_delta_phi, theta, m_radius );
			m_vertices[tempn].m_position = vector3f( polar3f( phi-m_delta_phi, theta, m_radius ) );
			m_vertices[tempn].m_color=color(0,0,0);

			m_indices[n]=tempn;
			n++;
		}
		m_num_vertices+=(int)needed;
	}
	m_num_vertices+=(int)needed_next;

// last triangle..

	m_vertices[m_num_vertices].m_polar.phi=phi;
	m_vertices[m_num_vertices].m_polar.theta=0;
	m_vertices[m_num_vertices].m_position=vector3f( polar3f(phi, 0, m_radius) );
	m_vertices[m_num_vertices].m_color=color(0,0,0);

	m_indices[n]=m_num_vertices;
	m_num_indices=n+1;
	m_num_vertices++;
}

void skydome::update()
{
	/* update skydome colors */
	update_colors();
	/* and update global colors */
	update_global_colors();
}

void skydome::update_colors()
{
	color base_color, atmosphere_part, haze_part, def_color;

	float p = 0;
	float ah, hh;
	float sundist, corona;
	float old_phi=-10;
	int i;

	color background_color = m_current_atmosphere_color/16.0f;

	if(get_atmosphere_light() || get_haze_light() || m_current_sun_intensity) // check of de zon schijnt ...
	{
		for (i=0; i < m_num_vertices; i++)
		{
			m_vertices[i]=m_vertices[i];

			if(m_vertices[i].m_polar.phi!=old_phi)
			{
				old_phi=m_vertices[i].m_polar.phi;

				p = m_vertices[i].m_polar.phi/90;
				ah = expf(-p*(1.0f/m_current_atmosphere_height))*m_current_atmosphere_density;
				hh = expf(-p*(1.0f/m_current_haze_height))*m_current_haze_density;
	
				// de basiskleur van de atmosphere_ bestaat uit een gradient blauw ...
				atmosphere_part = (m_current_atmosphere_color*ah);

				atmosphere_part = atmosphere_part * get_red_shift(hh);
				atmosphere_part = atmosphere_part * get_atmosphere_light();

				// deze basis kleur krijgt een 'laagje' mist

				haze_part = (m_current_haze_color*hh);

				// het laagje mist kan rood kleuren door een ondergaande zon ...
				// dit gebeurt met een 'redshift'

				haze_part = haze_part * get_red_shift(hh);
				haze_part = haze_part * get_haze_light();

				// en tenslotte tellen we dit op...

				base_color=atmosphere_part+haze_part;
			}

			sundist = (m_vertices[i].m_position-m_current_sun_position).get_length() / m_radius;

			corona = expf( (-sundist) / m_current_haze_height) * get_haze_light();
			corona += expf( (-sundist*80.0f) / m_current_haze_height);

			corona = corona * m_current_sun_intensity * m_current_haze_density; // belichting zon...

			if(corona<0) corona=0;
			def_color=base_color+((get_red_shift(hh) * m_current_haze_color) * corona);

			m_vertices[i].m_color=def_color+background_color;
			m_vertices[i].m_color.exposure(2.0f);
		}
	}
	else	// the sun isn't shining...
	{
		for (i=0; i < m_num_vertices; i++)
		{
			m_vertices[i].m_color=background_color;
		}
	}
}

void skydome::render(renderer_base* renderer, camera_base* camera)
{
	renderer->use_camera_rotation(camera);
	renderer->render_triangle_strip(m_vertices, m_indices, m_num_indices);
}

void skydome::set_atmosphere(color new_color, float new_height, float new_density)
{
	m_current_atmosphere_color = new_color;
	m_current_atmosphere_height = new_height;
	m_current_atmosphere_density = new_density;
}

void skydome::set_haze(color new_color, float new_height, float new_density)
{
	m_current_haze_color = new_color;
	m_current_haze_height = new_height;
	m_current_haze_density = new_density;
}

void skydome::set_decay(color new_color, float new_height, float new_density)
{
	m_current_decay_color = new_color;
	m_current_decay_height = new_height;
	m_current_decay_density = new_density;
}

void skydome::set_sun(color new_color, float new_intensity, polar3f new_polar)
{
	m_current_sun_color = new_color;
	m_current_sun_intensity = new_intensity;
	m_current_sun_position = vector3f( new_polar ).get_normalized() * m_radius;
	m_current_sun_polar = new_polar;
}

color skydome::get_red_shift(float shift)
{
	color RedShift = m_current_sun_color;
	color RedKey;
	color Unitcolor = color( 1.0f, 1.0f, 1.0f );

	if(shift<0.5f) RedKey = color( 1.0f, 1.0f-shift, 1.0f-2*shift );
	else RedKey = color( 1.0f, 1.0f-shift, 0.0f );

	RedShift = RedShift * RedKey;

	if(RedShift.r<0) RedShift.r=0;
	if(RedShift.g<0) RedShift.g=0;
	if(RedShift.b<0) RedShift.b=0;

	return (RedShift*get_red_shift_density()) + (Unitcolor*(1-get_red_shift_density()));
}


float skydome::get_red_shift_density()
{
	// de mate waarin er een 'redshift' optreedt hangt af van de hoek die
	// de zon op de atmosphere_ maakt ...
	// ik kies nu ff voor 15 graden

	float angle=8;

	if(fabs(m_current_sun_polar.phi)<=angle)
	{
		return wCosf(90.0f*fabsf(m_current_sun_polar.phi)/angle);
	}
	else return 0; 
}

float skydome::get_haze_light()
{
	// angle geeft aan vanaf waar de belichting moet beginnen ...
	float angle = -1;
	// angle_full geeft aan bij welke angle de belichting volledig moet zijn..
	float angle_full = 3;

	float phi=m_current_sun_polar.phi;

	if(phi>angle_full)
	{	// volledige belichting
		return 1;
	}
	else if(phi>angle && phi<angle_full)
	{	// interpolate
		return wCosf(90*(angle_full-phi)/(angle_full-angle));
	}
	else
	{	// return 0;
		return 0;
	}
}

float skydome::get_atmosphere_light()
{
	// angle geeft aan vanaf waar de belichting moet beginnen ...
	float angle = 0;
	// angle_full geeft aan bij welke angle de belichting volledig moet zijn..
	float angle_full = 10;

	float phi=m_current_sun_polar.phi;

	if(phi>angle_full)
	{	// volledige belichting
		return 1;
	}
	else if(phi>angle && phi<angle_full)
	{	// interpolate
		return wCosf(90*(angle_full-phi)/(angle_full-angle));
	}
	else
	{	// return 0;
		return 0;
	}
}

void skydome::update_global_colors()
{
	m_fog_color = m_vertices[m_indices[m_num_indices-1]].m_color;

	// met deze kleuren kan ook iets gezegd worden over de zon kleur
	m_sun_color = ((m_current_sun_color * get_red_shift(m_current_haze_density) * m_current_sun_intensity +
		     m_current_sun_color * get_atmosphere_light() * m_current_sun_intensity)*4);

	m_sun_flare_color = (m_sun_color/2);

	m_sun_flare_color.exposure( 2.0f );
	m_sun_color.exposure( 2.0f );

	// en die van de fog ..
	// dit is de kleur die de haze_ heeft bij de horizon ...

	m_fog_color = ((m_current_haze_color) +
		    (m_current_haze_color * get_red_shift(m_current_haze_density)) );

	m_fog_color.exposure( 2.0f );

	// nu de kleur voor de wolken ..

	m_cloud_color = m_current_haze_color * get_red_shift(m_current_haze_density) * (get_haze_light() * 0.5f + get_atmosphere_light());
	m_cloud_color.exposure( 2.0f );
	m_cloud_color += (m_current_atmosphere_color/24.0f);
}
