#ifndef _SKYDOME_
#define _SKYDOME_ 1

#include "../framework.h"

class skydome
{
public:
	skydome(float radius, int delta_theta, int delta_phi);
	~skydome();

	void update();
	void render(renderer_base* renderer, camera_base* camera);

	void set_atmosphere(color new_color, float new_height, float new_density);
	void set_haze(color new_color, float new_height, float new_density);
	void set_decay(color new_color, float new_height, float new_density);
	void set_sun(color new_color, float new_intensity, polar3f new_polar);

	color get_sun_color() {return m_sun_color; }
	color get_sun_flare_color() {return m_sun_flare_color; }
	color get_fog_color() {return m_fog_color; }
	color get_cloud_color() {return m_cloud_color; }
private:
	void generate_dome();
	void update_colors();
	void update_global_colors();

	color get_red_shift(float shift);
	float get_red_shift_density();
	float get_haze_light();
	float get_atmosphere_light();

	int m_num_vertices;
	vertex_color_polar *m_vertices;

	int m_num_indices;
	unsigned short *m_indices;

	int m_delta_theta;
	int m_delta_phi;
	float m_radius;

	color m_atmosphere_color, m_haze_color, m_decay_color;
	float m_atmosphere_height, m_haze_height, m_decay_height;
	float m_atmosphere_density, m_haze_density, m_decay_density;

	color m_current_atmosphere_color, m_current_haze_color, m_current_decay_color;
	float m_current_atmosphere_height, m_current_haze_height, m_current_decay_height;
	float m_current_atmosphere_density, m_current_haze_density, m_current_decay_density;
	
	color m_current_sun_color;
	float m_current_sun_intensity;
	vector3f m_current_sun_position;
	polar3f m_current_sun_polar;

	color m_sun_color; 
	color m_sun_flare_color;
	color m_fog_color;
	color m_cloud_color;
};

#endif /* _SKYDOME_ */
